! Copyright (C) 2019-2021:
!   Max Planck Institut fuer extraterrestrische Physik, Leibniz Institute for Astrophysics Potsdam (AIP)
!
! This file is part of eRosita's ESASS (https://erosita.mpe.mpg.de/). It is free software; you can
! redistribute it and/or modify it under the terms of the GNU General
! Public License as published by the Free Software Foundation; either
! version 2 of the License, or (at your option) any later version.
!
! This program is distributed in the hope that it will be useful,
! but WITHOUT ANY WARRANTY; without even the implied warranty of
! MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
! General Public License for more details.
!
! You should have received a copy of the GNU General Public License
! along with this library; if not, write to the Free Software
! Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
! USA, or see https://www.gnu.org/licenses/.
!

program catprep_main
    use param

    use catprep_mod
    use catprep_srclist_mod
    use catprep_mllist_mod

    use erolib_general_mod
    use erolib_file_mod
    use erolib_str_mod

    implicit none

    character(*), parameter :: function = "catprep_main"

    integer :: status, ierr
    integer :: ii

    type(tCatprep_merge) :: task
    type(tCatSrcList) :: CatSrcList ! product
    type(tCatSrcList) :: CatSrcList_in, CatSrcList_tmp

    status = ok

      ! read/open the .par file
    call opast(status)
    if(status /= ok) call wrstop(function, "error read par file")

    ! read the input paramters
    call wrlog(function, "reading parameters")
    call task%read_params(status)
    ! validate the input parameters
    call wrlog(function, "validate parameters")
    call task%validate(status)
    ! sanity check
    if(status /= ok) then
        call wrstop(function, "input parameters are invalid")
    else
        call wrlog(function, "parameters - valid")
    end if

    do ii = 1, task%n_infile
        call CatSrcList_in%open(task%infile(ii), readonly, status)
        call CatSrcList_in%read(status, extend=.true.)
        call CatSrcList_in%close(status)

        if(ii == 1) then
            CatSrcList = CatSrcList_in
        else
            CatSrcList_tmp = CatSrcList
            call merge_CatSrcList(CatSrcList_tmp, CatSrcList_in, CatSrcList, status)
        end if

        call CatSrcList_tmp%free(status)
        call CatSrcList_in%free(status)
    end do

    if(status == ok) then
        call wrlog(function, "create outfile")
        call CatSrcList%open(task%outfile, create, status)
        call CatSrcList%write(status)
        call CatSrcList%close(status)
        call CatSrcList%free(status)
    end if

    call cpast (status, ierr)

end program catprep_main
