! Copyright (C) 2019-2021:
!   Max Planck Institut fuer extraterrestrische Physik, Leibniz Institute for Astrophysics Potsdam (AIP)
!
! This file is part of eRosita's ESASS (https://erosita.mpe.mpg.de/). It is free software; you can
! redistribute it and/or modify it under the terms of the GNU General
! Public License as published by the Free Software Foundation; either
! version 2 of the License, or (at your option) any later version.
!
! This program is distributed in the hope that it will be useful,
! but WITHOUT ANY WARRANTY; without even the implied warranty of
! MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
! General Public License for more details.
!
! You should have received a copy of the GNU General Public License
! along with this library; if not, write to the Free Software
! Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
! USA, or see https://www.gnu.org/licenses/.
!

program catprep_main
  use param

  use catprep_mod
  use catprep_srclist_mod
  !    use catprep_srcconf_mod
  use catprep_mllist_mod

  use erolib_general_mod
  use erolib_str_mod

  implicit none

  character(*), parameter :: function = "catprep_main"

  integer :: status, ierr

  type(tCatprep) :: task
  type(tCatSrcList) :: CatSrcList ! product
  character(buffsize) :: pipeline_mode

  status = ok

    ! read/open the .par file
  call opast(status)
  if(status /= ok) call wrstop(function, "error read par file")

  ! read the input paramters
  call wrlog(function, "reading parameters")
  call task%read_params(status)
  ! validate the input parameters
  call wrlog(function, "validate parameters")
  call task%validate(status)
  ! sanity check
  if(status /= ok) then
    call wrstop(function, "input parameters are invalid")
  else
    call wrlog(function, "parameters - valid")
  end if
  
  call CatSrcList%init(0, status)

  ! fill the CatSrcList type with the provided input file
  call wrlog(function, "import infile")
  call insert_mllist_catsrclist(task%infile, task%det_alg_str, task%detuid_prefix, CatSrcList, status)

  if(CatSrcList%nelements > 0) then
    call wrlog(function, "set tstart/tstop columns")
    CatSrcList%tstart%value = CatSrcList%stdhdr%tstart
    CatSrcList%tstop%value = CatSrcList%stdhdr%tstop
    ! calculate the distance_NN and distance_X
    call CatSrcList%calc_srcdensity(task%p_srcdensity, status)
  end if

  ! write the CatSrcList to HDD
  ! create the new CatSrcList File
  call wrlog(function, "create outfile")
  if(task%clobber .eqv. .true.) then
    call CatSrcList%open(task%outfile, create, status, true)
  else
    call CatSrcList%open(task%outfile, create, status)
  end if
  call wrlog(function, "write / close new catalog")
  call CatSrcList%write(status)
  call CatSrcList%close(status)

  if(task%do_outfile2 .eqv. .true.) then
    call wrlog(function, "write / close new optional ownerflag filtered catalog")

    if(task%clobber .eqv. .true.) then
      call CatSrcList%open(task%outfile2, create, status, true)
    else
      call CatSrcList%open(task%outfile2, create, status)
    end if

    if(task%do_owner_filter .eqv. .true.) then
      call CatSrcList%apply_filter_ownerflag(task%owner, status)
    end if
    if(task%do_flux_filter .eqv. .true.) then
      call CatSrcList%apply_filter_flux(task%flux_threshold, status)
    end if
    call CatSrcList%write(status)
    call CatSrcList%close(status)
  end if

  call CatSrcList%free(status)
  
  call cpast (status, ierr)

end program catprep_main
